Unit Duketrig;
{ Trigonometry utilities for Duke3D }

{ angles, 0 is east, 2048 = 1 turn clockwise }
{ displacement +x=right, +y=bottom, +z=down  }
{ vertical angle + is slope down, hinge at firstwall }
{ v angle = vert/horiz *16 }

Interface
Uses Dukedefs,Dukeutil;

Function angle(w: integer): integer;
Function normalise_angle(a: integer): integer;
Function separation (a,b: longint): real;
Procedure midpoint(w: integer; var xmid,ymid: longint);
Function point_angle(w: integer; xp,yp: longint):integer;
Function wall_size (w: integer): longint;
Procedure extrapolate (w: integer; rad: longint; var newx,newy: longint);
Procedure rotate(var x1,y1: longint; theta, axis: integer);
Procedure mid_sector(n: integer; var sumx,sumy: longint);
Function inside(s: integer; x,y: longint): boolean;

Implementation

function sign(a: longint):integer;
{ returns 1 or -1 according to sign of a }
begin if a<0 then sign:=-1 else sign:=1; end;


Function arctangent(x,y: longint):integer;
{ returns angle in dukespace units of displacement x,y }
{ 0 is east, 2048 = 1 turn clockwise, x=right, y=down  }
begin
   if y=0 then
      arctangent:=-512*sign(x)+512
   else
      arctangent:=round(1024-2048*arctan(x/y)/2/pi-512*sign(y));
end;

Function angle(w: integer): integer;
{ returns wall angle in duke space units }
begin
   with wall[w]^ do
      angle:=arctangent(wall[point2]^.x-x,wall[point2]^.y-y);
end;

Function normalise_angle(a: integer): integer;
{ ensures angle is always in range 0-2047 }
begin
   normalise_angle:= a and $7FF;
end;

Function separation (a,b: longint): real;
{ returns euclidean separation of points }
var aa,bb: real;
begin
  aa:=a;
  bb:=b;
  if (a>65536) or (a<-65536) or (b>65536) or (b<-65536) then
     writeln(a,b);

  separation:=Round(sqrt(sqr(aa)+sqr(bb)));
end;

Procedure Midpoint(w: integer; var xmid,ymid: longint);
{ returns midpoint of nominated wall }
begin
   with wall[w]^ do begin
     xmid:=(x+wall[point2]^.x) div 2;
     ymid:=(y+wall[point2]^.y) div 2;
   end;
end;

Function point_angle(w: integer; xp,yp: longint):integer;
{ returns the angle vs normal between wall midpoint and a given point }
var xm,ym : longint;
    a: integer;
begin
   midpoint(w,xm,ym);
   point_angle:= normalise_angle (arctangent(xp-xm,yp-ym)-angle(w)-512);
end;

Function wall_size (w: integer): longint;
{ returns length of wall in duke pixels }
begin
  with wall[w]^ do
     wall_size:=round(separation(wall[point2]^.x-x,wall[point2]^.y-y));
end;

Procedure Extrapolate (w: integer; rad: longint; var newx,newy: longint);
{ in polar co-ordinates, extrapolate wall w by length rad, returns newx,newy }
var myfrac: real;
    xdist,ydist: longint;
begin
  with wall[w]^ do begin
     xdist:=wall[point2]^.x-x;
     ydist:=wall[point2]^.y-y;
     myfrac:=rad/separation(xdist,ydist);
     newx:=x+round(xdist*myfrac);
     newy:=y+round(ydist*myfrac);
  end;
end;

function sine(a: integer):real;
{ converts duke space angle to trigonometric sine }
begin
   sine:=sin(-a/325.9);
end;

Procedure rotate(var x1,y1: longint; theta, axis: integer);
{ rotates point x1,y1 through angle theta about the start of wall axis }
var phi: integer;
    rad: real;
    dx,dy: longint;
begin
   with wall[axis]^ do begin
      dx:=x1-x;
      dy:=y1-y;
      phi:=arctangent(dx,dy) + theta;
      rad:=separation(x1-x,y1-y);
      x1:=round(sine(phi-512)*rad+x);
      y1:=round(-sine(phi)*rad+y);
   end;
end;



Procedure Mid_sector(n: integer; var sumx,sumy: longint);
{ find centre of sector n }
var w,i : integer;
begin
   sumx:=0;
   sumy:=0;
   with sector[n]^ do begin
      w:=wallptr;
      for i:=0 to wallnum-1 do
         with wall[w]^ do begin
           sumx:=sumx+x;
           sumy:=sumy+y;
           w:=point2;
         end;
      sumx:=sumx div longint (wallnum);
      sumy:=sumy div longint (wallnum);
   end;
end;

Function inside(s: integer; x,y: longint): boolean;
{ returns true if point x,y lies inside sector z }
{ this algorithm only works for convex polygons }
var w,p: integer;
begin
   inside:=false;
   with sector[s]^ do begin
      w:=wallptr;
      repeat
         p:=point_angle(w,x,y);
         if (p>512) and (p<1536) then exit;
         w:=wall[w]^.point2;
      until w=wallptr;
   end;
   inside:=true;
end;

begin
end.